(* Tm3rUrlLabel 1.21
**
** Target: C++ Builder/Delphi 32 bit
**
** Copyright (c) 2000 Jacob Dybala (m3Rlin). Freeware.
**   <m3rlin@programmer.net>
** Portions (c) 1998 Ray Adams (TRUrlLabel)
**
** Created : March 12 2000
** Modified: April 14 2000
**
** History
** 1.0 [March 12 2000]
**   Initial version.
** 1.1 [April 3 2000]
**   + Added CreateUrl() and SetUrl() function.
**   + Added ShowUrlHint property.
**   * Default font name changed from MS Sans Serif to Verdana.
**   * Now Tm3rUrlLabel is always underlined if DetectMouse is False.
** 1.2 [April 8 2000]
**   + Added MailBody property.
**   + 6 new URL types added:
**     utFile, utGopher, utHttps, utNews, utTelnet and utWais.
**   * Now Hint property is updated after mail subject change.
**   * Fixed CreatUrl() bug. If no subject was specified it still would leave
**     the '?Subject=' part.
**   * 'ut' prefix added to URL types.
** 1.21 [April 14 2000]
**   + Added Verdana font presence check on create.
**
** To do: Multiple body lines: #13#10 = %0A%0D. Use TStrings?, hints seem to be
**   a bit wrong.
**
**   Please e-mail me what youi would like to see in this component or any bug
** reports.
**
** License:
**   You have the right to use this component in freeware, public domain,
** shareware and commercial programs under one term: please include my name
** [Jacob Dybala (m3Rlin)] in the AboutBox or in the documentation.
**   No warrenties of any kind are given.
*)

unit UrlLabel;

interface

uses
  Classes, Graphics, Controls, Forms, StdCtrls, Messages;

type
  TUrlType = (
    utCustom, utFile, utFtp, utGopher, utHttp, utHttps, utMail, utNews,
    utTelnet, utWais
  );

  Tm3rUrlLabel = class(TLabel)
  private
    fDetectMouse,
    fShowUrlHint  : Boolean;
    fCustomUrlType,
    fMailBody,
    fMailSubject,
    fURL          : string;
    fActiveColor,
    fInactiveColor: TColor;
    fOnMouseEnter,
    fOnMouseLeave : TNotifyEvent;
    fUrlType      : TUrlType;
    function CreateUrl: string;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
  protected
    procedure SetDetectMouse(Value: Boolean);
    procedure SetMailBody(Value: string);
    procedure SetMailSubject(Value: string);
    procedure SetUrl(Value: string);
    procedure SetUrlHint(Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
  published
    property ActiveColor: TColor read fActiveColor write fActiveColor
      default clBlue;
    property CustomURLType: string read fCustomUrlType write fCustomUrlType;
    property DetectMouse: Boolean read fDetectMouse write SetDetectMouse;
    property InactiveColor: TColor read fInactiveColor write fInactiveColor
      default clNavy;
    property MailBody: string read fMailBody write SetMailBody;
    property MailSubject: string read fMailSubject write SetMailSubject;
    property ShowUrlHint: Boolean read fShowUrlHint write SetUrlHint
      default True;
    property URLType: TUrlType read fUrlType write fUrlType default utHttp;
    property URL: string read fURL write SetURL;

    property OnMouseEnter: TNotifyEvent read fOnMouseEnter write fOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read fOnMouseLeave write fOnMouseLeave;
  end;

procedure Register;

implementation

uses
  ShellApi, Windows;

{ Tm3rUrlLabel }

constructor Tm3rUrlLabel.Create(AOwner: TComponent);
const
  sVERDANA: 'Verdana';
begin
  inherited;
  with Font do begin
    Color := clNavy;
    if Screen.Fonts.IndexOf(sVERDANA) <> -1 then
      Name := sVERDANA
    else
      Name := 'Arial';    
  end;
  Cursor := crHandPoint;
  fDetectMouse := True;
  ActiveColor := clBlue;
  InactiveColor := clNavy;
  ShowUrlHint := True;
end;

procedure Tm3rUrlLabel.CMMouseEnter(var Msg: TMessage);
begin
  if fDetectMouse then begin
    Font.Color := fActiveColor;
    Font.Style := [fsUnderline];
  end;
  inherited;
  if Assigned(fOnMouseEnter) then
    fOnMouseEnter(Self);
end;

procedure Tm3rUrlLabel.CMMouseLeave(var Msg: TMessage);
begin
  if fDetectMouse then begin
    Font.Color := fInActiveColor;
    Font.Style := [];
  end;
  inherited;
  if Assigned(fOnMouseLeave) then
    fOnMouseLeave(Self);
end;

procedure Tm3rUrlLabel.SetDetectMouse(Value: Boolean);
begin
  fDetectMouse := Value;
  if Value then
    Font.Style := Font.Style - [fsUnderline]
  else
    Font.Style := Font.Style + [fsUnderline];
end;

procedure Tm3rUrlLabel.SetMailBody(Value: string);
begin
  fMailBody := Value;
  if fShowUrlHint then
    Hint := CreateUrl;
end;

procedure Tm3rUrlLabel.SetMailSubject(Value: string);
begin
  fMailSubject := Value;
  if fShowUrlHint then
    Hint := CreateUrl;
end;

procedure Tm3rUrlLabel.SetUrl(Value: string);
begin
  fUrl := Value;
  if fShowUrlHint then
    Hint := CreateUrl;
end;

procedure Tm3rUrlLabel.SetUrlHint(Value: Boolean);
begin
  fShowUrlHint := Value;
  if Value then
    Hint := CreateUrl
  else if Hint = CreateUrl then
    Hint := '';
end;

function Tm3rUrlLabel.CreateUrl: string;
begin
  case fUrlType of
    utCustom: Result := fCustomUrlType + fURL;
    utFile: Result := 'file://' + fURL;
    utFtp: Result := 'ftp://' + fURL;
    utGopher: Result := 'gopher://' + fURL;
    utHttp: Result := 'http://' + fURL;
    utHttps: Result := 'https://' + fURL;
    utMail: begin
      Result := 'mailto:' + fURL;
      if fMailSubject <> '' then
        Result := Result + '?Subject=' + fMailSubject;
      if fMailBody <> '' then
        Result := Result + '&Body=' + fMailBody;
    end;
    utNews: Result := 'news:' + fURL;
    utTelnet: Result := 'telnet:' + fURL;
    utWais: Result := 'wais:' + fURL;
  end;
end;

procedure Tm3rUrlLabel.Click;
begin
  ShellExecute(GetDesktopWindow(), 'open', PChar(CreateUrl), nil, nil,
    SW_SHOWNORMAL);
end;

procedure Register;
begin
  RegisterComponents('m3Rlin Web', [Tm3rUrlLabel]);
end;

end.
